/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 * 
 * Contributor(s):
 * 
 * Portions Copyrighted 2008 Sun Microsystems, Inc.
 */

package org.netbeans.modules.wsdlextensions.jms.configeditor;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.netbeans.modules.wsdlextensions.jms.JMSAddress;
import org.netbeans.modules.wsdlextensions.jms.JMSBinding;
import org.netbeans.modules.wsdlextensions.jms.JMSConstants;
import org.netbeans.modules.wsdlextensions.jms.JMSMessage;
import org.netbeans.modules.wsdlextensions.jms.JMSOperation;
import org.netbeans.modules.xml.schema.model.GlobalElement;
import org.netbeans.modules.xml.schema.model.GlobalType;
import org.netbeans.modules.xml.wsdl.model.Binding;
import org.netbeans.modules.xml.wsdl.model.BindingInput;
import org.netbeans.modules.xml.wsdl.model.BindingOperation;
import org.netbeans.modules.xml.wsdl.model.BindingOutput;
import org.netbeans.modules.xml.wsdl.model.Message;
import org.netbeans.modules.xml.wsdl.model.Part;
import org.netbeans.modules.xml.wsdl.model.Port;
import org.netbeans.modules.xml.wsdl.model.WSDLComponent;
import org.netbeans.modules.xml.wsdl.model.WSDLModel;

/**
 * Controller to allow for population of the model to the
 * visual components as well as persistence of the model
 * 
 * @author jalmero
 */
public class InboundPersistenceController {
    private InboundOneWayMessagePanel jmsEditorPanel = null;
    private JMSAdvancedPanel mAdvPanel = null;
    private WSDLComponent mWSDLComponent = null;
    private InboundPersistenceController mController = null;
    private String mMsgName = "JMSInputMessage"; // NOI18N
    private boolean mSolicited = false;
        
    public InboundPersistenceController(WSDLComponent modelComponent,
            InboundOneWayMessagePanel visualComponent) {
        this(modelComponent, visualComponent, false);
    }
    
    public InboundPersistenceController(WSDLComponent modelComponent,
            InboundOneWayMessagePanel visualComponent, boolean isSolicited) {
        jmsEditorPanel = visualComponent;
        mWSDLComponent = modelComponent;
        mSolicited = isSolicited;
        mController = this;
        if (mSolicited) {
            mMsgName = "SolicitedReadOutputMessage"; // NOI18N
        }
    }   
    
    public InboundPersistenceController(WSDLComponent modelComponent,
            JMSAdvancedPanel visualComponent) {
        mAdvPanel = visualComponent;
        mWSDLComponent = modelComponent;
        mController = this;
    }
    
    public boolean commit() {
        if (mWSDLComponent instanceof JMSAddress) {
            return commitAddress((JMSAddress) mWSDLComponent);
        } else if (mWSDLComponent instanceof JMSBinding) {
            return commitBinding((JMSBinding) mWSDLComponent);
        } else if (mWSDLComponent instanceof Port) {
            return commitPort((Port) mWSDLComponent);
        } else if (mWSDLComponent instanceof JMSMessage) {
            return commitMessage((JMSMessage) mWSDLComponent);
        } else if (mWSDLComponent instanceof JMSOperation) {
            return commitOperation((JMSOperation) mWSDLComponent);
        }
        return false;
    }

    public boolean rollback() {
        return true;
    }

    private boolean commitAddress(JMSAddress jmsAddress) {
        WSDLModel wsdlModel = mWSDLComponent.getModel();
        try {
            if (!wsdlModel.isIntransaction()) {
               wsdlModel.startTransaction(); 
            }                 
            commitServices(jmsAddress);     
            Port port = (Port) jmsAddress.getParent();
            Binding binding = port.getBinding().get();
            Collection<BindingOperation> bindingOperations =
                    binding.getBindingOperations();
            String operationName = "";
            if (jmsEditorPanel != null) {
                operationName = jmsEditorPanel.getOperationName();
            }
            commitBindingOperations(binding, bindingOperations, operationName);
        } catch (Exception ex) {
            ex.printStackTrace(System.err);
            return false;
        } 
        return true;
    }
    
    private boolean commitBinding(JMSBinding jmsBinding) {
        WSDLModel wsdlModel = jmsBinding.getModel();
        try {                        
            if (!wsdlModel.isIntransaction()) {
               wsdlModel.startTransaction(); 
            }           
            JMSAddress jmsAddress = null;
            if (jmsEditorPanel != null) {
                jmsAddress = jmsEditorPanel.
                        getJMSAddressPerSelectedPort();
            }
            commitServices(jmsAddress);
            Binding binding = (Binding) jmsBinding.getParent();
            Collection<BindingOperation> bindingOperations =
                    binding.getBindingOperations();
            String operationName = "";
            if (jmsEditorPanel != null) {
                operationName = jmsEditorPanel.getOperationName();
            }
            commitBindingOperations(binding, bindingOperations, operationName);          
        } catch (Exception ex) {
            ex.printStackTrace(System.err);
            return false;
        }
        return true;
    }    
    
    private void commitJMSOperation(JMSOperation jmsOp) {
        if (jmsOp != null) {
            // From JMS Connection Section
            // moved to DestinationPanel
        }     
    }
    
    private void commitInputJMSMessage(Binding binding, String opName,
            JMSMessage jmsMessage) {
        if (jmsMessage != null) {
            if (jmsEditorPanel != null) {
                // Message Type Section
//                jmsMessage.setMessageType(jmsEditorPanel.
//                        getInputMessageType());
//                jmsMessage.setTextPart(jmsEditorPanel.
//                        getInputMessageText());
                if (jmsMessage.getMessageType() != null) {
                    if (!jmsMessage.getMessageType().equals(jmsEditorPanel.getInputMessageType())) {
                        jmsMessage.setMessageType(jmsEditorPanel.getInputMessageType());
                    }
                } else {
                    if (jmsEditorPanel.getInputMessageType() != null) {
                        jmsMessage.setMessageType(jmsEditorPanel.getInputMessageType());
                    }
                } 

                if (jmsEditorPanel.getInputUse().equals(JMSConstants.ENCODED)) {
                    jmsMessage.setJMSEncodingStyle(jmsEditorPanel.getInputEncodingStyle());
                    jmsMessage.setUse(jmsEditorPanel.getInputUse());
                } else {
                    if ((jmsMessage.getUse() != null) &&
                            (jmsEditorPanel.getInputUse().equals(JMSConstants.LITERAL))) {
                        jmsMessage.setJMSEncodingStyle(null);
                        jmsMessage.setUse(null);
                    }
                }     

                if (jmsEditorPanel.getForwardAsAttachment()) {
                    jmsMessage.setForwardAsAttachment(jmsEditorPanel.
                            getForwardAsAttachment());
                } else {
                    jmsMessage.setAttribute(JMSMessage.ATTR_FORWARD_AS_ATTACHMENT,
                        null);            
                }  
                
                // set the type of the message part if it is undefined based
                // on the MessageType option from user
                commitMessageType(binding, opName, jmsMessage);  
            } 
        }
    }    
    
    public boolean commitServices(JMSAddress jmsAddress) {
        if (jmsAddress != null) {
            // From JMS Connection Section
            if (jmsEditorPanel != null) {
                jmsAddress.setConnectionURL(jmsEditorPanel.getConnectionURL());
                jmsAddress.setUsername(jmsEditorPanel.getUserName());
                jmsAddress.setPassword(jmsEditorPanel.getPassword());
            }
            
            // per meeting 08/14/08, values for jndi info below will be
            // persisted/retrieved from a single string attribute.  runtime
            // will be responsible for getting correct info from here  
            if (mAdvPanel != null) {
                // just get the free-form string from text area
                String jndiInfoStr = mAdvPanel.getText();
                JMSUtilities.createJMSJCAOption(jmsAddress, jndiInfoStr);
//            jmsAddress.setConnectionFactoryName(jmsEditorPanel.getConnectionFactoryName());
//            jmsAddress.setInitialContextFactory(jmsEditorPanel.getInitialContextFactory());
//            jmsAddress.setProviderURL(jmsEditorPanel.getProviderURL());
//            jmsAddress.setSecurityPrincipal(jmsEditorPanel.getSecurityPrincipal());
//            jmsAddress.setSecurityCredentials(jmsEditorPanel.getSecurityCredentials());
            }
        }
        return true;
    }
    
   private boolean commitPort(Port port) {
        Collection<JMSAddress> address = port.
                getExtensibilityElements(JMSAddress.class);
        JMSAddress jmsAddress = address.iterator().next();
        return commitAddress(jmsAddress);
    }
    
    private boolean commitMessage(JMSMessage jmsMessage) {
        Object parentObj = jmsMessage.getParent();
        BindingOperation parentOp = null;
        if (parentObj instanceof BindingInput) {
            parentOp = (BindingOperation) ((BindingInput) parentObj).
                    getParent();
        } else if (parentObj instanceof  BindingOutput) {
            parentOp = (BindingOperation) ((BindingOutput) parentObj).
                    getParent();
        }
        if (parentObj != null) {
            Binding parentBinding = (Binding) parentOp.getParent();
            Collection<JMSBinding> bindings = parentBinding.
                    getExtensibilityElements(JMSBinding.class);
            if (!bindings.isEmpty()) {
                return commitBinding(bindings.iterator().next());
            }
        }
        return false;
    }
     
    private boolean commitOperation(JMSOperation fileOperation) {
        Object obj = fileOperation.getParent();
        if (obj instanceof BindingOperation) {
            Binding parentBinding = (Binding) ((BindingOperation) obj).getParent();
            Collection<JMSBinding> bindings = parentBinding.
                    getExtensibilityElements(JMSBinding.class);
            if (!bindings.isEmpty()) {
                return commitBinding(bindings.iterator().next());
            }
        }
        return false;
    }    
    
    private void commitBindingOperations(Binding binding, 
            Collection<BindingOperation> bops,
            String operationName) {
        if (bops != null) {
           for (BindingOperation bop : bops) {
                if (bop.getName().equals(operationName)) {
                    List<JMSOperation> jmsOpsList = bop.
                            getExtensibilityElements(JMSOperation.class);
                    Iterator<JMSOperation> jmsOps =
                            jmsOpsList.iterator();
                    // there should only be one jms:operation for the binding op
                    if (jmsOpsList.size() > 0) {
                        JMSOperation jmsOp = jmsOps.next();
                        
                        // save the jms operation info
                        commitJMSOperation(jmsOp);
                    }
                    
                    // save the jms input message info (output if solicited)
                    JMSMessage msgToUpdate = null;
                    if (mSolicited) {
                        BindingOutput bo = bop.getBindingOutput();
                        List<JMSMessage> outputJMSMessages =
                                bo.getExtensibilityElements(JMSMessage.class);
                        msgToUpdate = outputJMSMessages.get(0);                                                
                    } else {
                        BindingInput bi = bop.getBindingInput();
                        List<JMSMessage> inputJMSMessages =
                                bi.getExtensibilityElements(JMSMessage.class);
                        msgToUpdate = inputJMSMessages.get(0);                        
                       
                    }
                    commitInputJMSMessage(binding, operationName,
                            msgToUpdate); 
                }
            }            
        }
    }

    private void commitMessageType(Binding binding, String opName,
            JMSMessage jmsMessage) {
        String partName = jmsEditorPanel.getInputMessageText();
        GlobalType gT = jmsEditorPanel.getSelectedPartType();
        GlobalElement gE = jmsEditorPanel.getSelectedElementType();         
        if (partName != null) {
            // set input part (output part if solicited)
            Collection parts = new ArrayList();
            if (mSolicited) {
                parts = JMSUtilities.getOutputParts(binding, opName);
            } else {
                parts = JMSUtilities.getInputParts(binding, opName);
            }
            if ((parts != null) && (parts.size() > 0)) {
                Iterator iter = parts.iterator();
                while (iter.hasNext()) {
                    Part partEntry = (Part) iter.next();
                    if (partEntry.getName().equals(partName)) {
                        JMSUtilities.setPartType(partEntry, 
                                jmsEditorPanel.getMessageType(), gT, gE);
                    } 
                }
            }
            JMSUtilities.setPart(jmsEditorPanel.getMessageType(),
                    jmsMessage, partName);                                 
            
            // if solicited, need to also update the inbound as well
            if (mSolicited) {
                parts = JMSUtilities.getInputParts(binding, opName);
                if ((parts != null) && (parts.size() > 0)) {
                    Iterator iter = parts.iterator();
                    while (iter.hasNext()) {
                        Part partEntry = (Part) iter.next();
                        if (partEntry.getName().equals(partName)) {
                            JMSUtilities.setPartType(partEntry, 
                                    jmsEditorPanel.getMessageType(), gT, gE);
                        } 
                    }
                }            
            }            
        } else {
                            
            // need to create a message with part
            if (!mSolicited) {
                Message newMessageIn = JMSUtilities.
                        createMessage(mWSDLComponent.getModel(), mMsgName); 
                if (newMessageIn != null) {
                    Part newPart = JMSUtilities.
                            createPartAndSetType(mWSDLComponent.getModel(),
                            newMessageIn, "part1", jmsEditorPanel.getMessageType(),
                            gT, gE);
                    if (newPart != null) {
                        JMSUtilities.setPart(jmsEditorPanel.getMessageType(), 
                                jmsMessage, newPart.getName());                         
                    }
                }
            }
            
            if (mSolicited) {
                // handle the inbound as well but not set to binding section
//                Message newMessageIn = JMSUtilities.
//                        createMessage(mWSDLComponent.getModel(),
//                        "SolicitedReadInputMessage"); // NOI18N
//                if (newMessageIn != null) {
//                    Part newPart = JMSUtilities.createPartAndSetType(
//                            mWSDLComponent.getModel(), newMessageIn, "part1",
//                            jmsEditorPanel.getMessageType(), gT, gE);
//                }
                
                // handle the outbound 
                Message newMessageOut = JMSUtilities.
                        createMessage(mWSDLComponent.getModel(), 
                        "SolicitedReadOutputMessage"); // NOI18N
                if (newMessageOut != null) {
                    Part newPart = JMSUtilities.createPartAndSetType(
                            mWSDLComponent.getModel(), newMessageOut, "part1", 
                            jmsEditorPanel.getMessageType(), gT, gE);
                    if (newPart != null) {
                        JMSUtilities.setPart(jmsEditorPanel.getMessageType(), 
                                jmsMessage, newPart.getName());                   
                    }                    
                }                 
            }
        }
    }
    
    public static void populate() {
        
    }
    
    public void setPartType(Part part, int messageType) {
        GlobalType gT = jmsEditorPanel.getSelectedPartType();
        GlobalElement gE = jmsEditorPanel.getSelectedElementType();        
        if (messageType == JMSConstants.XML_MESSAGE_TYPE) {
            if (gT != null) {
                part.setType(part.createSchemaReference(
                        gT, GlobalType.class));
                part.setElement(null);       
            } else if (gE != null) {
                part.setElement(part.createSchemaReference(
                        gE, GlobalElement.class));
                part.setType(null);               
            }
        } else if (messageType ==
                JMSConstants.TEXT_MESSAGE_TYPE) {
            part.setType(part.createSchemaReference(
                    JMSUtilities.getPrimitiveType("string"),
                    GlobalType.class));
            part.setElement(null);
        } else if (messageType ==
                JMSConstants.ENCODED_MESSAGE_TYPE) {
            if (gT != null) {
                part.setType(part.createSchemaReference(
                        gT, GlobalType.class));
                part.setElement(null);
            } else if (gE != null) {
                part.setType(null);
                part.setElement(part.createSchemaReference(
                        gE, GlobalElement.class));
            }
        }  else if (messageType ==
                JMSConstants.BINARY_MESSAGE_TYPE) {
            part.setType(part.createSchemaReference(
                    JMSUtilities.getPrimitiveType(JMSConstants.BASE64_BINARY),
                    GlobalType.class));
            part.setElement(null);
        } 
    }
    
    public void setAdvancedPanel(JMSAdvancedPanel panel) {
        mAdvPanel = panel;
    }
}
